#!/usr/bin/python3

import os,re,time,getpass,subprocess,markdown
from string import Template
from argparse import ArgumentParser

########################################
# UI - check for arguments and options #
########################################

parser = ArgumentParser(description="Manage your weblog via commandline. Creates a static html5 compliant website with your publications.")

# general options
parser.add_argument('-d','--blog-dir', dest="blog_dir", default='~/public_html/blog', help='path to blog directory (default: %(default)s)')

### subroutines of pb
subparsers = parser.add_subparsers(help="targets of operation", dest='target')

# 'blog' command
parser_blog = subparsers.add_parser('blog', help='add or update blog or its sections')
## 'blog' subroutines
subparsers_blog = parser_blog.add_subparsers(help="blog subcommands", dest='action')
blog_parser_update = subparsers_blog.add_parser('update', help='updates blog to current state')
blog_parser_update.add_argument('subsection', choices=['all','recent','archive','feed'], help='sections of blog you may want to update')
blog_parser_add = subparsers_blog.add_parser("add", help='add an non-existing blog at --blog-dir')

# 'draft' command
parser_draft = subparsers.add_parser('draft', help='create, list, edit, publish or remove drafts')
## 'draft' subroutines
subparsers_draft = parser_draft.add_subparsers(help="draft subcommands", dest='action')
draft_parser_add = subparsers_draft.add_parser('add', help='add a new draft')
draft_parser_add.add_argument('name', help='define an alphanumeric name')
draft_parser_list = subparsers_draft.add_parser('list', help='list drafts` names')
draft_parser_edit = subparsers_draft.add_parser('edit', help='edit certain draft')
draft_parser_edit.add_argument('name', help='name like specified by [list]')
draft_parser_remove = subparsers_draft.add_parser('remove', help='remove certain draft')
draft_parser_remove.add_argument('name', help='name like specified by [list]')
draft_parser_publish = subparsers_draft.add_parser('publish', help='publish draft as article')
draft_parser_publish.add_argument('name', help='name of draft to publish')

# 'article' command
parser_article = subparsers.add_parser('article', help='import, list, edit or delete article')

## 'article' subroutines
subparsers_article = parser_article.add_subparsers(help="article subcommands", dest='action')
article_parser_add = subparsers_article.add_parser('add', help='add a new article')
article_parser_add.add_argument('name', help='define an alphanumeric name')
article_parser_list = subparsers_article.add_parser('list', help='list articles')
article_parser_edit = subparsers_article.add_parser('edit', help='edit certain article')
article_parser_edit.add_argument('id', help='id like specified by [list]')
article_parser_remove = subparsers_article.add_parser('remove', help='remove certain article')
article_parser_remove.add_argument('id', help='id like specified by [list]')


args = parser.parse_args()
print(args)

####################
# Global Constants #
####################

# logged in user
user = getpass.getuser()

# set path to blog
try:
    blog_dir = os.path.abspath(os.path.expanduser(vars(args)['blog_dir']))
    if not os.path.isdir(blog_dir):
        print("The path you offered is not a directory!")
        exit
except:
    raise

(pbconf,blogconf,l10nconf) = (False,False,False)

# path to blog's templates
tplDir = os.path.join(blog_dir,"templates")

# path to drafts
draftDir = os.path.join(blog_dir,"drafts")

# paths to configs
pbconfpath = os.path.join("/","etc","picoblogger","pb.conf") # global config
blogconfpath = os.path.join(blog_dir,"pb.conf") # local config



##################
# Help Functions #
##################

# parse unix-config-style files

def parse_conf(path):
    aVars = {}
    if os.path.exists(path):
        f = open(path, "r")
    else:
        f = path.split("\n")
    for line in f:
        if "#" in line:
            line = line[0:line.find("#")]
        pattern = '^(\w+)\s*?[=|:]\s*(.+)[^#]*$'
        match = re.search(pattern, line)
        if match:
            aVars[match.group(1)] = match.group(2)
    return aVars



######################
# Parse Config Files #
######################

try:
    if os.path.isfile(pbconfpath):
        pbconf = parse_conf(pbconfpath)
except:
    print("Couldn't parse systems global picoblogger configuration (%s)."%pbconfpath)
    raise


try:
    if os.path.isfile(blogconfpath):
        blogconf = parse_conf(blogconfpath)

except:
    print("Couldn't parse blog's local configuration (%s)."%blogconfpath)
    raise

l10nconfpath = os.path.join(blog_dir,"l10n",blogconf['BLOG_LANG'],"static.conf") # localization file
try:
    if os.path.isfile(l10nconfpath):
        l10nconf = parse_conf(l10nconfpath)
except:
    print("Couldn't parse localization file (%s)."%l10nconfpath)
    raise


########################
# Templating functions #
########################

def parse_article(path):
    article = open(path,"r").read()
    header = article.split("HEADER_BEGIN")[1]
    header = header.split("HEADER_END")[0]
    body = markdown.markdown(article.split("HEADER_END")[1])
    article_dict = parse_conf(header)
    article_dict['BODY'] = body
    article_dict['MODIFIED_DATE'] = time.strftime("%x",time.gmtime(os.stat(path).st_mtime))
    article_dict['MODIFIED_TIME'] = time.strftime("%H:%M:%S",time.gmtime(os.stat(path).st_mtime))

    return article_dict

def article_to_html(article_dict):
    sARTICLE = open(os.path.join(tplDir,"article.htm")).read()
    sARTICLE = Template(sARTICLE).safe_substitute(l10nconf)

    # Read out article txt file
    
    tplSub = {
        "ARTICLE_TITLE":article_dict['TITLE'],
        "ARTICLE_AUTHOR":article_dict['AUTHOR'],
        "ARTICLE_CREATED_DATE":article_dict['DATE'],
        "ARTICLE_CREATED_TIME":article_dict['TIME'],
        "ARTICLE_MODIFIED_DATE":article_dict['MODIFIED_DATE'],
        "ARTICLE_MODIFIED_TIME":article_dict['MODIFIED_TIME'],
        "ARTICLE_BODY":article_dict['BODY'],
        "ARTICLE_TAGS":article_dict['TAGS']
    }
        
    sARTICLE = Template(sARTICLE).safe_substitute(tplSub)
    return sARTICLE

def join_articles():
    artDir = os.path.join(blog_dir,"articles")
    joined_html = ""
    articles_dict = {}
    for name in [os.path.splitext(item)[0] for item in os.listdir(draftDir) if not "\~" in item]:
        article = (parse_article(os.path.join(artDir, name + ".txt")))
        # Bring articles into chronological order
        t = time.mktime(time.strptime(article['DATE'] + " " + article['TIME'],"%m/%d/%y %H:%M:%S"))
        articles_dict[str(t)] = article
    dates_of_creation = [value for value in articles_dict.keys()]
    print(dates_of_creation)
    for date in dates_of_creation:
        joined_html += article_to_html(articles_dict[str(time.strftime(date))])
    
    return joined_html
    
def update_blog(subsection):
    # Joins config defined variables into templates
    
    tplIndex = os.path.join(tplDir,"index.htm")
    blogIndex = os.path.join(blog_dir,"index.html")

    tpl = Template(open(tplIndex,'r').read())
    tmpIndex = tpl.safe_substitute(l10nconf)

    if blogconf:
        tmpIndex = Template(tmpIndex).safe_substitute(blogconf)
    if pbconf:
        tmpIndex = Template(tmpIndex).safe_substitute(pbconf)

    sNAV = open(os.path.join(tplDir,"nav.htm")).read()
    sNAV = Template(sNAV).safe_substitute(l10nconf)
#    sMAIN = open(os.path.join(tplDir,"main.htm")).read()
#    sMAIN = Template(sMAIN).safe_substitute(join_articles())
    sMAIN = join_articles()
    sASIDE = open(os.path.join(tplDir,"aside.htm")).read()
    sFOOTER = open(os.path.join(tplDir,"footer.htm")).read()

    tplSub = {
        "TEMPLATE_NAV":sNAV,
        "TEMPLATE_MAIN":sMAIN,
        "TEMPLATE_ASIDE":sASIDE,
        "TEMPLATE_FOOTER":sFOOTER
        }

    tmpIndex = Template(tmpIndex).safe_substitute(tplSub)
    
    with open(blogIndex,"w", encoding=l10nconf['BLOG_CHARSET']) as f:
        print(tmpIndex, file=f)

# def article_to_html(name):
#     article = os.path.join(blog_dir,"articles",name + ".txt")
#     txt = open(article).read()
#     html = markdown.markdown(txt)
#     return html

        
##################
# Main Functions #
##################

# blog functions

if vars(args)['target'] == 'blog':

    
    ## update subsections or whole blog

    if vars(args)['action'] == 'update':

        if vars(args)['subsection'] in ('recent','all'):
            pass

        if vars(args)['subsection'] in ('archive','all'):
            pass

        if vars(args)['subsection'] in ('feed','all'):
            pass
        
        # Joining all templates
        update_blog(vars(args)['subsection'])
        print("Updated your blog.")        



    ## add a new blog at -d blog-directory

    if vars(args)['action'] == 'add':
        pass


# draft functions

if vars(args)['target'] == 'draft':


    ## add new draft

    if vars(args)['action'] == 'add':
        draft = os.path.join(blog_dir,"drafts",vars(args)['name'] + ".txt")
        tpl = os.path.join(tplDir,"article.tpl")
        sTpl = open(tpl).read()
        subs = {"DATE" : time.strftime("%x",time.localtime()), "TIME" : time.strftime("%X",time.localtime()),
                "AUTHOR" : user
                }
        
        sTpl = Template(sTpl).safe_substitute(subs)
        with open(draft,"w", encoding=l10nconf['BLOG_CHARSET']) as f:
            print(sTpl, file=f)
        
        subprocess.call(["emacs", "-nw", draft])


    ## list drafts

    if vars(args)['action'] == 'list':
        [print(item) for item in [os.path.splitext(item)[0] for item in os.listdir(draftDir) if not "~" in item]]


        
    ## edit draft

    if vars(args)['action'] == 'edit':
        draft = os.path.join(blog_dir,"drafts",vars(args)['name'] + ".txt")
        subprocess.call(["emacs", " -nw", draft])

    ## remove draft

    if vars(args)['action'] == 'remove':
        draft = os.path.join(blog_dir,"drafts",vars(args)['name'] + ".txt")
        try:
            os.remove(draft)
        except:
            print("Can't find %s."%draft)

    ## publish draft as article

    if vars(args)['action'] == 'publish':
        draft = os.path.join(blog_dir,"drafts",vars(args)['name'] + ".txt")
        article = os.path.join(blog_dir,"articles",vars(args)['name'] + ".txt")
        os.replace(draft, article)

# article functions

if vars(args)['target'] == 'article':


    ## add new article
    if vars(args)['action'] == 'add':
        pass

    
    ## list articles

    if vars(args)['action'] == 'list':
        pass


    ## edit article

    if vars(args)['action'] == 'edit':
        draft = os.path.join(blog_dir,"articles",vars(args)['id'] + ".txt")
        subprocess.call(["emacs", " -nw", draft])



    ## remove article

    if vars(args)['action'] == 'remove':
        pass

